from __future__ import with_statement
from __future__ import absolute_import
from __future__ import print_function
import sys


try:
    import numpy
    import reflex
    from pipeline_product import PipelineProduct
    import pipeline_display
    import reflex_plot_widgets
    import matplotlib.gridspec as gridspec
    from collections import defaultdict  # to make dictionary of lists
    import pdb  # for debugging
    import_success = True

except ImportError:
    import_success = False
    print("Error importing modules pyfits, wx, matplotlib, numpy")

# Median absolute deviation function; used to scale the images
def MAD(x):
    x=numpy.array(x)
    return numpy.median(numpy.abs(x-numpy.median(x)))

def paragraph(text, width=None):
    """ wrap text string into paragraph
       text:  text to format, removes leading space and newlines
       width: if not None, wraps text, not recommended for tooltips as
              they are wrapped by wxWidgets by default
    """
    import textwrap
    if width is None:
        return textwrap.dedent(text).replace('\n', ' ').strip()
    else:
        return textwrap.fill(textwrap.dedent(text), width=width)


class DataPlotterManager(object):
    """
    This class must be added to the PipelineInteractiveApp with setPlotManager
    It must have following member functions which will be called by the app:
     - setInteractiveParameters(self)
     - readFitsData(self, fitsFiles):
     - addSubplots(self, figure):
     - plotProductsGraphics(self, figure, canvas)
    Following members are optional:
     - setWindowHelp(self)
     - setWindowTitle(self)
    """

    # static members
    recipe_name = "vircam_linearity_analyse"
    mst_cat = "MASTER_CHANNEL_TABLE"

    def setWindowTitle(self):
        return self.recipe_name+"_interactive"

    def setInteractiveParameters(self):
        """
        This function specifies which are the parameters that should be presented
        in the window to be edited.  Note that the parameter has to also be in the
        in_sop port (otherwise it won't appear in the window). The descriptions are
        used to show a tooltip. They should match one to one with the parameter
        list.
        """
        return [

            reflex.RecipeParameter(recipe=self.recipe_name, displayName="nord",
                                   group="vircam_linearity_analyse", 
                                   description="Order of polynomial fit. [4]"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="lthr",
                                   group="vircam_linearity_analyse", 
                                   description="Lower bad pixel threshold. [8.0]"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="hthr",
                                   group="vircam_linearity_analyse", 
                                   description="Upper bad pixel threshold. [8.0]"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="maxbpmfr",
                                   group="vircam_linearity_analyse", 
                                   description="Maximum number of frames used in bpm analysis [10]"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="adjust",
                                   group="vircam_linearity_analyse", 
                                   description="Adjust stats with monitor set [TRUE]"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="diagnostic",
                                   group="vircam_linearity_analyse", 
                                   description="Write out diagnostic tables. [FALSE]"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="ext",
                                   group="vircam_linearity_analyse", 
                                   description="Extension number to be done, 0 == all. [1]")
            ]

    def readFitsData(self, fitsFiles):
        """
        This function should be used to read and organize the raw fits files
        produced by the recipes.
        It receives as input a list of reflex.FitsFiles
        """
        # organize the files into a dictionary, here we assume we only have 
        # one file per category if there are more, one must use a
        # dictionary of lists
        self.frames = dict()
        for f in fitsFiles:
            self.frames[f.category] = PipelineProduct(f)

        # we only have two states, we have data or we don't
        # define the plotting functions we want to use for each

        if self.mst_cat in self.frames:
            self.mst_tab = self.frames[self.mst_cat]
            self.mst_found = True

            # table is a list of FITS record arrays, one for each extension
            # table = self.mst_tab.all_hdu[1].data
            # access data by field name: table['COLNAME']
            # see help at https://pythonhosted.org/pyfits/users_guide/users_table.html


        else:
            # Set the plotting functions to NODATA ones
            self._add_subplots = self._add_nodata_subplots
            self._plot = self._nodata_plot

    def addSubplots(self, figure):

        # Make 1x1 grid of plots
        self.img_plot = []
        gs = gridspec.GridSpec(1, 1)
        self.img_plot.append(figure.add_subplot(gs[0,0]))

    def plotProductsGraphics(self):
        
        title = " " 
        tooltip = "Coloured number is chip number\nError bars are overlaid on each number"

        colours = ['red','blue','green','purple']

        mst_table = []
        for i in range(len(self.mst_tab.all_hdu)-1):
            mst_table.append(self.mst_tab.all_hdu[i+1].data)

        for i in range(1):
            self.img_plot[i].grid(True)
            self.img_plot[i].set_title(title, fontsize=12, fontweight='semibold')
            self.img_plot[i].tooltip = tooltip
            self.img_plot[i].set_aspect('auto')

        # Pull out correct numbers in table
        x=defaultdict(list)
        y=defaultdict(list)
        y_err=defaultdict(list)
        for j in range(len(mst_table)):
            x[j].append(mst_table[j]['channum'])
            y[j].append(mst_table[j]['lin_10000'])
            y_err[j].append(mst_table[j]['lin_10000_err'])

        self.img_plot[0].set_xlabel("Channel Number")
        self.img_plot[0].set_ylabel("Percentage non-linearity at 10000 cts")

        for i_row in range(len(mst_table)):
            self.img_plot[0].scatter(x[i_row][0], y[i_row][0], 120, color = colours[i_row%4],marker = r"$ {} $".format(i_row+1))
            self.img_plot[0].errorbar(numpy.asarray(x[i_row][0]), numpy.asarray(y[i_row][0]), yerr = numpy.asarray(y_err[i_row][0]),
                                      linestyle='none', color = colours[i_row%4], alpha = 0.5)

    def _add_nodata_subplots(self, figure):
        self.img_plot = figure.add_subplot(1,1,1)

    def _nodata_plot(self):
        # could be moved to reflex library?
        self.img_plot.set_axis_off()
        text_nodata = "Data not found. Input files should contain this" \
                       " type:\n%s" % self.mst_cat
        self.img_plot.text(0.1, 0.6, text_nodata, color='#11557c',
                      fontsize=18, ha='left', va='center', alpha=1.0)
        self.img_plot.tooltip = 'No data found'

    def setWindowHelp(self):
      help_text = """
This is an interactive window which help asses the quality of the execution of a recipe.
"""
      return help_text

#This is the 'main' function
if __name__ == '__main__':
    from reflex_interactive_app import PipelineInteractiveApp

    # Create interactive application
    interactive_app = PipelineInteractiveApp(enable_init_sop=True)

    # get inputs from the command line
    interactive_app.parse_args()

    #Check if import failed or not
    if not import_success:
        interactive_app.setEnableGUI(False)

    #Open the interactive window if enabled
    if interactive_app.isGUIEnabled():
        #Get the specific functions for this window
        dataPlotManager = DataPlotterManager()

        interactive_app.setPlotManager(dataPlotManager)
        interactive_app.showGUI()
    else:
        interactive_app.set_continue_mode()

    #Print outputs. This is parsed by the Reflex python actor to
    #get the results. Do not remove
    interactive_app.print_outputs()
    sys.exit()
