from __future__ import with_statement
from __future__ import absolute_import
from __future__ import print_function
import sys


try:
    import numpy
    import reflex
    from pipeline_product import PipelineProduct
    import pipeline_display
    import reflex_plot_widgets

    from matplotlib import gridspec, pylab, pyplot, transforms
    import pdb  # for debugging
    from collections import defaultdict  # to make dictionary of lists
    import_success = True
    
except ImportError:
    import_success = False
    print("Error importing modules pyfits, wx, matplotlib, numpy")

# Median absolute deviation function; used to scale the images
def MAD(x):
    x=numpy.array(x)
    return numpy.median(numpy.abs(x-numpy.median(x)))


def paragraph(text, width=None):
    """ wrap text string into paragraph
       text:  text to format, removes leading space and newlines
       width: if not None, wraps text, not recommended for tooltips as
              they are wrapped by wxWidgets by default
    """
    import textwrap
    if width is None:
        return textwrap.dedent(text).replace('\n', ' ').strip()
    else:
        return textwrap.fill(textwrap.dedent(text), width=width)


class DataPlotterManager(object):
    """
    This class must be added to the PipelineInteractiveApp with setPlotManager
    It must have following member functions which will be called by the app:
     - setInteractiveParameters(self)
     - readFitsData(self, fitsFiles):
     - addSubplots(self, figure):
     - plotProductsGraphics(self, figure, canvas)
    Following members are optional:
     - setWindowHelp(self)
     - setWindowTitle(self)
    """

    # static members
    
    recipe_name = "vircam_science_postprocess"
    tile_cat = "TILED_IMAGE"  #  tiled image

    def setWindowTitle(self):
        return self.recipe_name+"_interactive"

    def setInteractiveParameters(self):
        """
        This function specifies which are the parameters that should be presented
        in the window to be edited.  Note that the parameter has to also be in the
        in_sop port (otherwise it won't appear in the window). The descriptions are
        used to show a tooltip. They should match one to one with the parameter
        list.
        """

        # Only selected Recipe parameters are shown because list is too long
        # The selection shown ones likely to be wanted to be changed by user
        return [
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="nebulise",
                                   group="vircam_science_postprocess", 
                                   description="Nebulise stacks before object detection?. [TRUE]"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="grout",
                                   group="vircam_science_postprocess", 
                                   description="Grout the output tile catalogue? [TRUE]"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="minphotom",
                                   group="vircam_science_postprocess", 
                                   description="Minimum number of stars for photometry solution [20]"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="cdssearch",
                                   group="vircam_science_postprocess", 
                                   description="Use CDS for 2MASS standards?. [FALSE]"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="neb_medfilt",
                                   group="vircam_science_postprocess", 
                                   description="Median filter size for nebuliser. [101]"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="neb_linfilt",
                                   group="vircam_science_postprocess", 
                                   description="Median filter size for nebuliser. [33]"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="cat_ipix",
                                   group="vircam_science_postprocess", 
                                   description="Minimum pixel area for each detected object. [4]"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="cat_thresh",
                                   group="vircam_science_postprocess", 
                                   description="Detection threshold in sigma above sky. [1.25]"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="cat_icrowd",
                                   group="vircam_science_postprocess", 
                                   description="Use deblending?. [TRUE]"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="cat_rcore",
                                   group="vircam_science_postprocess", 
                                   description="Value of Rcore in pixels. [3.0]"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="cat_nbsize",
                                   group="vircam_science_postprocess", 
                                   description="Background smoothing box size. [64]"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="cacheloc",
                                   group="vircam_science_postprocess", 
                                   description="Location of standard star cache [.]"),
            reflex.RecipeParameter(recipe=self.recipe_name, displayName="magerrcut",
                                   group="vircam_science_postprocess", 
                                   description="Magnitude error cut [100.0]")
            ]

    def readFitsData(self, fitsFiles):
        """
        This function should be used to read and organize the raw fits files
        produced by the recipes.
        It receives as input a list of reflex.FitsFiles
        """

        # frames is a dict of keyword/list pairs where elements of list are PipelineProducts
        #  it contains all FITS files in the input parameter list

        self.tile_frames = []

        for f in fitsFiles:
            if (f.category == self.tile_cat):
                self.tile_frames.append(PipelineProduct(f))

        if (len(self.tile_frames)) == 1:  # should only ever be one tile!
            
            self.tile_img_found = True
            self.n_extn = len(self.tile_frames[0].hdulist())-1  # number of extensions, assumed to be same for all fitsFiles

            # re-define eso-rex's pipeline_display plotting functions to enable callbacks
            # (if we add radiobuttons in the future)
            self._add_subplots = self._add_subplots
            self._plot = self._data_plot

        elif (len(self.tile_frames)) > 1:
            print("Fatal error - more than one tile found")
            exit

        elif (len(self.tile_frames)) == 0:
            # Set the plotting functions to NODATA ones
            self._add_subplots = self._add_nodata_subplots
            self._plot = self._nodata_plot

    def addSubplots(self, figure):
        self._add_subplots(figure)

    def plotProductsGraphics(self):
        self._plot()

    def _add_subplots(self, figure):

        self.img_plot = []
        gs = gridspec.GridSpec(1, 1)
        self.img_plot.append(figure.add_subplot(gs[0,0]))
            
    def _data_plot(self):



        imgdisp = pipeline_display.ImageDisplay()
        imgdisp.setAspect('equal')
        self.img_plot[0].cla()

        pylab.setp(self.img_plot[0].get_xticklabels(), visible = True)
        imgdisp.setLabels('X','Y')

        try:
            filt_name = self.tile_frames[0].all_hdu[0].header['HIERARCH ESO INS FILT1 NAME']
        except:
            filt_name = ''

        title = "Tile {} ".format(filt_name)
        # Try reading image
        try:
            temp = self.tile_frames[0]
            temp.readImage(0)  # image is in primary HDU

            # Set z-limit using 1 iteration of sigma clipping using MED and MAD
            #temp_image = temp.image[numpy.isfinite(temp.image)]
            #med = numpy.median(temp_image)
            #mad = MAD(temp_image)
            #temp_image = temp_image[numpy.abs(temp_image-med) < 3*1.48*mad]
            #new_med = numpy.median(temp_image)
            #new_mad = MAD(temp_image)
            # Set limits to median-1sigma, median+10sigma ?
            #imgdisp.z_lim = new_med-1.48*new_mad, new_med+10*1.48*new_mad

            imgdisp.display(self.img_plot[0], title, "Tile:\n" + 
                            temp.fits_file.name, temp.image)
        except IndexError:
            self.img_plot[0].set_axis_off()
            text_nodata = "No tile found !"
            self.img_plot[0].text(0.1, 0.5, text_nodata, color='#11557c',
                                    fontsize=18, ha='left', va='center', alpha=1.0,
                                    transform = self.img_plot[0].transAxes)
            self.img_plot[0].tooltip = 'No data found'

    def _add_nodata_subplots(self, figure):
        self.img_plot = figure.add_subplot(1,1,1)

    def _nodata_plot(self):
        # could be moved to reflex library?
        self.img_plot.set_axis_off()
        text_nodata = "Data not found. Input files should contain this" \
                       " type:\n%s" % self.img_cat
        self.img_plot.text(0.1, 0.6, text_nodata, color='#11557c',
                      fontsize=18, ha='left', va='center', alpha=1.0)
        self.img_plot.tooltip = 'No data found'


    def setWindowHelp(self):
      help_text = """
This is an interactive window which help asses the quality of the execution of a recipe.
"""
      return help_text


#This is the 'main' function
if __name__ == '__main__':
    from reflex_interactive_app import PipelineInteractiveApp

    # Create interactive application
    interactive_app = PipelineInteractiveApp(enable_init_sop=True)

    # get inputs from the command line
    interactive_app.parse_args()

    #Check if import failed or not
    if not import_success:
        interactive_app.setEnableGUI(False)

    #Open the interactive window if enabled
    if interactive_app.isGUIEnabled():
        #Get the specific functions for this window
        dataPlotManager = DataPlotterManager()

        interactive_app.setPlotManager(dataPlotManager)
        interactive_app.showGUI()
    else:
        interactive_app.set_continue_mode()

    #Print outputs. This is parsed by the Reflex python actor to
    #get the results. Do not remove
    interactive_app.print_outputs()
    sys.exit()
